package scripts;

import com.stencyl.graphics.G;
import com.stencyl.graphics.BitmapWrapper;
import com.stencyl.graphics.ScaleMode;

import com.stencyl.behavior.Script;
import com.stencyl.behavior.Script.*;
import com.stencyl.behavior.ActorScript;
import com.stencyl.behavior.SceneScript;
import com.stencyl.behavior.TimedTask;

import com.stencyl.models.Actor;
import com.stencyl.models.GameModel;
import com.stencyl.models.actor.Animation;
import com.stencyl.models.actor.ActorType;
import com.stencyl.models.actor.Collision;
import com.stencyl.models.actor.Group;
import com.stencyl.models.Scene;
import com.stencyl.models.Sound;
import com.stencyl.models.Region;
import com.stencyl.models.Font;
import com.stencyl.models.Joystick;

import com.stencyl.Config;
import com.stencyl.Engine;
import com.stencyl.Input;
import com.stencyl.Key;
import com.stencyl.utils.Utils;

import openfl.ui.Mouse;
import openfl.display.Graphics;
import openfl.display.BlendMode;
import openfl.display.BitmapData;
import openfl.display.Bitmap;
import openfl.events.Event;
import openfl.events.KeyboardEvent;
import openfl.events.TouchEvent;
import openfl.net.URLLoader;

import box2D.common.math.B2Vec2;
import box2D.dynamics.B2Body;
import box2D.dynamics.B2Fixture;
import box2D.dynamics.joints.B2Joint;

import motion.Actuate;
import motion.easing.Back;
import motion.easing.Cubic;
import motion.easing.Elastic;
import motion.easing.Expo;
import motion.easing.Linear;
import motion.easing.Quad;
import motion.easing.Quart;
import motion.easing.Quint;
import motion.easing.Sine;

import com.stencyl.graphics.shaders.BasicShader;
import com.stencyl.graphics.shaders.GrayscaleShader;
import com.stencyl.graphics.shaders.SepiaShader;
import com.stencyl.graphics.shaders.InvertShader;
import com.stencyl.graphics.shaders.GrainShader;
import com.stencyl.graphics.shaders.ExternalShader;
import com.stencyl.graphics.shaders.InlineShader;
import com.stencyl.graphics.shaders.BlurShader;
import com.stencyl.graphics.shaders.SharpenShader;
import com.stencyl.graphics.shaders.ScanlineShader;
import com.stencyl.graphics.shaders.CSBShader;
import com.stencyl.graphics.shaders.HueShader;
import com.stencyl.graphics.shaders.TintShader;
import com.stencyl.graphics.shaders.BloomShader;



class Design_2_2_FlickeronDamage extends ActorScript
{
	public var _Invincible:Bool;
	public var _InvincibleTime:Float;
	public var _HandleFlicker:Bool;
	public var _HandleInvincibility:Bool;
	public var _FlickerTime:Float;
	public var _FlickerOpacity:Float;
	public var _FlickerState:Float;
	public var _FlickerMode:String;
	public var _TintColor:Int;
	public var _IsDefending:Bool;
	public var _FlickerwhenDefending:Bool;
	
	/* ========================= Custom Event ========================= */
	public function _customEvent_Get():Void
	{
		if((actor.hasBehavior("Defend") && actor.isBehaviorEnabled("Defend")))
		{
			_IsDefending = asBoolean(actor.getValue("Defend", "_IsDefending"));
		}
	}
	public function _customEvent_Flicker():Void
	{
		if((_FlickerwhenDefending || (!(_FlickerwhenDefending) && !(_IsDefending))))
		{
			_Invincible = true;
		}
	}
	
	
	public function new(dummy:Int, actor:Actor, dummy2:Engine)
	{
		super(actor);
		nameMap.set("Actor", "actor");
		nameMap.set("Invincible", "_Invincible");
		_Invincible = false;
		nameMap.set("Invincible Time", "_InvincibleTime");
		_InvincibleTime = 0.0;
		nameMap.set("Handle Flicker", "_HandleFlicker");
		_HandleFlicker = false;
		nameMap.set("Handle Invincibility", "_HandleInvincibility");
		_HandleInvincibility = false;
		nameMap.set("Flicker Time", "_FlickerTime");
		_FlickerTime = 0.0;
		nameMap.set("Flicker Opacity", "_FlickerOpacity");
		_FlickerOpacity = 100.0;
		nameMap.set("Flicker State", "_FlickerState");
		_FlickerState = 0.0;
		nameMap.set("Flicker Mode", "_FlickerMode");
		_FlickerMode = "";
		nameMap.set("Tint Color", "_TintColor");
		_TintColor = Utils.getColorRGB(255,0,0);
		nameMap.set("Is Defending", "_IsDefending");
		_IsDefending = false;
		nameMap.set("Flicker when Defending", "_FlickerwhenDefending");
		_FlickerwhenDefending = false;
		
	}
	
	override public function init()
	{
		
		/* ======================== When Updating ========================= */
		addWhenUpdatedListener(null, function(elapsedTime:Float, list:Array<Dynamic>):Void
		{
			if(wrapper.enabled)
			{
				if((_FlickerState > 1))
				{
					_FlickerState = 0;
				}
				if(_Invincible)
				{
					/* Invincibility */
					if(!(_HandleInvincibility))
					{
						_HandleInvincibility = true;
						runLater(1000 * _InvincibleTime, function(timeTask:TimedTask):Void
						{
							_HandleInvincibility = false;
							_Invincible = false;
							actor.fadeTo(100 / 100, 0, Quad.easeIn);
							actor.clearFilters();
						}, actor);
					}
					/* Flicker */
					if(!(_HandleFlicker))
					{
						_HandleFlicker = true;
						if((_FlickerState == 0))
						{
							if((_FlickerMode == "Opacity"))
							{
								actor.fadeTo(_FlickerOpacity / 100, 0, Quad.easeOut);
							}
							if((_FlickerMode == "Tint"))
							{
								actor.setFilter([createTintFilter(_TintColor, _FlickerOpacity/100)]);
							}
							if((_FlickerMode == "Brightness"))
							{
								actor.setFilter([createBrightnessFilter(_FlickerOpacity)]);
							}
						}
						if((_FlickerState == 1))
						{
							if((_FlickerMode == "Opacity"))
							{
								actor.fadeTo(100 / 100, 0, Quad.easeOut);
							}
							if(((_FlickerMode == "Tint") || (_FlickerMode == "Brightness")))
							{
								actor.clearFilters();
							}
						}
						runLater(1000 * _FlickerTime, function(timeTask:TimedTask):Void
						{
							_FlickerState += 1;
							_HandleFlicker = false;
						}, actor);
					}
				}
				else
				{
					_FlickerState = 0;
				}
			}
		});
		
	}
	
	override public function forwardMessage(msg:String)
	{
		
	}
}